@@ -98,7 +98,7 @@ public:
 	enum eMergeStrategy { FAST_MERGE, DEEP_MERGE };
 
 	// FWD
-	class SortingComparator;
+	class CTPP2DECL SortingComparator;
 
 	/**
 	  @brief Constructor
@@ -231,6 +231,13 @@ public:
 	*/
 	CDT & operator[](const UINT_32  iPos);
 
+        /**
+	  @brief Provides access to the data contained in CDT (constant method)
+	  @param iPos - The index of the element
+	  @return Object with data
+	*/
+	const CDT & operator[](const UINT_32  iPos) const;
+
 	/**
 	  @brief Provides access to the data contained in CDT
 	  @param sKey - The key of the element
@@ -238,6 +245,13 @@ public:
 	*/
 	CDT & operator[](const STLW::string & sKey);
 
+	/**
+	  @brief Provides access to the data contained in CDT (constant method)
+	  @param sKey - The key of the element
+	  @return Object with data
+	*/
+	const CDT & operator[](const STLW::string & sKey) const;
+
 	/**
 	  @brief Provides constant access to the data contained in CDT
 	  @param iPos - The index of the element
@@ -1732,7 +1746,7 @@ public:
 	  @class SortingComparator CDT.hpp <CDT.hpp>
 	  @brief Sorting comparator
 	*/
-	class SortingComparator
+	class CTPP2DECL SortingComparator
 	{
 	public:
 		/**
@@ -1755,13 +1769,13 @@ public:
 	};
 
 	// FWD
-	class ConstIterator;
+	class CTPP2DECL ConstIterator;
 
 	/**
 	  @class Iterator CDT.hpp <CDT.hpp>
 	  @brief CDT[HASH] forward iterator
 	*/
-	class Iterator
+	class CTPP2DECL Iterator
 	{
 	private:
 		friend class CDT;
@@ -1840,7 +1854,7 @@ public:
 	  @class ConstIterator CDT.hpp <CDT.hpp>
 	  @brief CDT[HASH] forward constant iterator
 	*/
-	class ConstIterator
+	class CTPP2DECL ConstIterator
 	{
 	private:
 		friend class CDT;
@@ -1935,13 +1949,13 @@ public:
 
 private:
 	// Friends
-	friend CDT operator-(const UINT_64 & oValue, const CDT & oCDT);
-	friend CDT operator-(const INT_64  & oValue, const CDT & oCDT);
-	friend CDT operator-(const W_FLOAT & oValue, const CDT & oCDT);
+	CTPP2DECL friend CDT operator-(const UINT_64 & oValue, const CDT & oCDT);
+	CTPP2DECL friend CDT operator-(const INT_64  & oValue, const CDT & oCDT);
+	CTPP2DECL friend CDT operator-(const W_FLOAT & oValue, const CDT & oCDT);
 
-	friend CDT operator/(const UINT_64 & oValue, const CDT & oCDT);
-	friend CDT operator/(const INT_64  & oValue, const CDT & oCDT);
-	friend CDT operator/(const W_FLOAT & oValue, const CDT & oCDT);
+	CTPP2DECL friend CDT operator/(const UINT_64 & oValue, const CDT & oCDT);
+	CTPP2DECL friend CDT operator/(const INT_64  & oValue, const CDT & oCDT);
+	CTPP2DECL friend CDT operator/(const W_FLOAT & oValue, const CDT & oCDT);
 
 	// FWD
 	struct _CDT;
@@ -2008,7 +2022,7 @@ private:
   @param oCDT - 2-nd argument
   @return result of addition with same type of oCDT
 */
-CDT operator+(const UINT_64 & oValue, const CDT & oCDT);
+CTPP2DECL CDT operator+(const UINT_64 & oValue, const CDT & oCDT);
 
 /**
   @brief Operator + for INT_64 and CDT arguments
@@ -2016,7 +2030,7 @@ CDT operator+(const UINT_64 & oValue, const CDT & oCDT);
   @param oCDT - 2-nd argument
   @return result of addition with same type of oCDT
 */
-CDT operator+(const INT_64 & oValue, const CDT & oCDT);
+CTPP2DECL CDT operator+(const INT_64 & oValue, const CDT & oCDT);
 
 /**
   @brief Operator + for UINT_32 and CDT arguments
@@ -2024,7 +2038,7 @@ CDT operator+(const INT_64 & oValue, const CDT & oCDT);
   @param oCDT - 2-nd argument
   @return result of addition with same type of oCDT
 */
-CDT operator+(const UINT_32 & oValue, const CDT & oCDT);
+CTPP2DECL CDT operator+(const UINT_32 & oValue, const CDT & oCDT);
 
 /**
   @brief Operator + for INT_32 and CDT arguments
@@ -2032,7 +2046,7 @@ CDT operator+(const UINT_32 & oValue, const CDT & oCDT);
   @param oCDT - 2-nd argument
   @return result of addition with same type of oCDT
 */
-CDT operator+(const INT_32 & oValue, const CDT & oCDT);
+CTPP2DECL CDT operator+(const INT_32 & oValue, const CDT & oCDT);
 
 /**
   @brief Operator + for W_FLOAT and CDT arguments
@@ -2040,7 +2054,7 @@ CDT operator+(const INT_32 & oValue, const CDT & oCDT);
   @param oCDT - 2-nd argument
   @return result of addition with same type of oCDT
 */
-CDT operator+(const W_FLOAT & oValue, const CDT & oCDT);
+CTPP2DECL CDT operator+(const W_FLOAT & oValue, const CDT & oCDT);
 
 // ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
@@ -2050,7 +2064,7 @@ CDT operator+(const W_FLOAT & oValue, const CDT & oCDT);
   @param oCDT - 2-nd argument
   @return result of substraction with same type of oCDT
 */
-CDT operator-(const UINT_64 & oValue, const CDT & oCDT);
+CTPP2DECL CDT operator-(const UINT_64 & oValue, const CDT & oCDT);
 
 /**
   @brief Operator - for INT_64 and CDT arguments
@@ -2058,7 +2072,7 @@ CDT operator-(const UINT_64 & oValue, const CDT & oCDT);
   @param oCDT - 2-nd argument
   @return result of substraction with same type of oCDT
 */
-CDT operator-(const INT_64 & oValue, const CDT & oCDT);
+CTPP2DECL CDT operator-(const INT_64 & oValue, const CDT & oCDT);
 
 /**
   @brief Operator - for UINT_32 and CDT arguments
@@ -2066,7 +2080,7 @@ CDT operator-(const INT_64 & oValue, const CDT & oCDT);
   @param oCDT - 2-nd argument
   @return result of substraction with same type of oCDT
 */
-CDT operator-(const UINT_32 & oValue, const CDT & oCDT);
+CTPP2DECL CDT operator-(const UINT_32 & oValue, const CDT & oCDT);
 
 /**
   @brief Operator - for INT_32 and CDT arguments
@@ -2074,7 +2088,7 @@ CDT operator-(const UINT_32 & oValue, const CDT & oCDT);
   @param oCDT - 2-nd argument
   @return result of substraction with same type of oCDT
 */
-CDT operator-(const INT_32 & oValue, const CDT & oCDT);
+CTPP2DECL CDT operator-(const INT_32 & oValue, const CDT & oCDT);
 
 /**
   @brief Operator - for W_FLOAT and CDT arguments
@@ -2082,7 +2096,7 @@ CDT operator-(const INT_32 & oValue, const CDT & oCDT);
   @param oCDT - 2-nd argument
   @return result of substraction with same type of oCDT
 */
-CDT operator-(const W_FLOAT & oValue, const CDT & oCDT);
+CTPP2DECL CDT operator-(const W_FLOAT & oValue, const CDT & oCDT);
 
 // ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
@@ -2092,7 +2106,7 @@ CDT operator-(const W_FLOAT & oValue, const CDT & oCDT);
   @param oCDT - 2-nd argument
   @return result of multiplication with same type of oCDT
 */
-CDT operator*(const UINT_64 & oValue, const CDT & oCDT);
+CTPP2DECL CDT operator*(const UINT_64 & oValue, const CDT & oCDT);
 
 /**
   @brief Operator * for INT_64 and CDT arguments
@@ -2100,7 +2114,7 @@ CDT operator*(const UINT_64 & oValue, const CDT & oCDT);
   @param oCDT - 2-nd argument
   @return result of multiplication with same type of oCDT
 */
-CDT operator*(const INT_64 & oValue, const CDT & oCDT);
+CTPP2DECL CDT operator*(const INT_64 & oValue, const CDT & oCDT);
 
 /**
   @brief Operator * for UINT_32 and CDT arguments
@@ -2108,7 +2122,7 @@ CDT operator*(const INT_64 & oValue, const CDT & oCDT);
   @param oCDT - 2-nd argument
   @return result of multiplication with same type of oCDT
 */
-CDT operator*(const UINT_32 & oValue, const CDT & oCDT);
+CTPP2DECL CDT operator*(const UINT_32 & oValue, const CDT & oCDT);
 
 /**
   @brief Operator * for INT_32 and CDT arguments
@@ -2116,7 +2130,7 @@ CDT operator*(const UINT_32 & oValue, const CDT & oCDT);
   @param oCDT - 2-nd argument
   @return result of multiplication with same type of oCDT
 */
-CDT operator*(const INT_32 & oValue, const CDT & oCDT);
+CTPP2DECL CDT operator*(const INT_32 & oValue, const CDT & oCDT);
 
 /**
   @brief Operator * for W_FLOAT and CDT arguments
@@ -2124,7 +2138,7 @@ CDT operator*(const INT_32 & oValue, const CDT & oCDT);
   @param oCDT - 2-nd argument
   @return result of multiplication with same type of oCDT
 */
-CDT operator*(const W_FLOAT & oValue, const CDT & oCDT);
+CTPP2DECL CDT operator*(const W_FLOAT & oValue, const CDT & oCDT);
 
 // ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
@@ -2134,7 +2148,7 @@ CDT operator*(const W_FLOAT & oValue, const CDT & oCDT);
   @param oCDT - 2-nd argument
   @return result of division with same type of oCDT
 */
-CDT operator/(const UINT_64 & oValue, const CDT & oCDT);
+CTPP2DECL CDT operator/(const UINT_64 & oValue, const CDT & oCDT);
 
 /**
   @brief Operator / for INT_64 and CDT arguments
@@ -2142,7 +2156,7 @@ CDT operator/(const UINT_64 & oValue, const CDT & oCDT);
   @param oCDT - 2-nd argument
   @return result of division with same type of oCDT
 */
-CDT operator/(const INT_64 & oValue, const CDT & oCDT);
+CTPP2DECL CDT operator/(const INT_64 & oValue, const CDT & oCDT);
 
 /**
   @brief Operator / for UINT_32 and CDT arguments
@@ -2150,7 +2164,7 @@ CDT operator/(const INT_64 & oValue, const CDT & oCDT);
   @param oCDT - 2-nd argument
   @return result of division with same type of oCDT
 */
-CDT operator/(const UINT_32 & oValue, const CDT & oCDT);
+CTPP2DECL CDT operator/(const UINT_32 & oValue, const CDT & oCDT);
 
 /**
   @brief Operator / for INT_32 and CDT arguments
@@ -2158,7 +2172,7 @@ CDT operator/(const UINT_32 & oValue, const CDT & oCDT);
   @param oCDT - 2-nd argument
   @return result of division with same type of oCDT
 */
-CDT operator/(const INT_32 & oValue, const CDT & oCDT);
+CTPP2DECL CDT operator/(const INT_32 & oValue, const CDT & oCDT);
 
 /**
   @brief Operator / for W_FLOAT and CDT arguments
@@ -2166,7 +2180,7 @@ CDT operator/(const INT_32 & oValue, const CDT & oCDT);
   @param oCDT - 2-nd argument
   @return result of division with same type of oCDT
 */
-CDT operator/(const W_FLOAT & oValue, const CDT & oCDT);
+CTPP2DECL CDT operator/(const W_FLOAT & oValue, const CDT & oCDT);
 
 } // namespace CTPP
 #endif // _CDT_HPP__
