@@ -129,32 +129,6 @@ public:
 	                   const UINT_32        iScopeNameLength,
 	                   const VMDebugInfo  & oDebugInfo = VMDebugInfo());
 
-	/**
-	  @brief Change visibility scope of CDT data (for ARRAY iterating)
-	  @param szNS - namespace
-	  @param iNSLength - namespace length
-	  @param szName - variable name (everything after namespace)
-	  @param iNameLength - variable name length
-	  @param szFullVariable - full variable name (ns + name)
-	  @param iFullVariableLength -  full variable name length
-	  @param oDebugInfo - debug information object
-	  @return instruction pointer if success, -1 if any error occured
-	*/
-	INT_32 ChangeForeachScope(CCHAR_P              szNS,
-	                          const UINT_32        iNSLength,
-	                          CCHAR_P              szName,
-	                          const UINT_32        iNameLength,
-	                          CCHAR_P              szFullVariable,
-	                          const UINT_32        iFullVariableLength,
-	                          const VMDebugInfo  & oDebugInfo);
-
-	/**
-	  @brief Change visibility scope of CDT data (for ARRAY-of-ARRAYS iterating)
-	  @param oDebugInfo - debug information object
-	  @return instruction pointer if success, -1 if any error occured
-	*/
-	INT_32 ChangeContextScope(const VMDebugInfo & oDebugInfo);
-
 	/**
 	  @brief Reset scope to previous CDT data (for ARRAY iterating)
 	  @param iIP - instruction pointer
@@ -164,15 +138,6 @@ public:
 	INT_32 ResetScope(const UINT_32        iIP,
 	                  const VMDebugInfo  & oDebugInfo = VMDebugInfo());
 
-	/**
-	  @brief Reset scope to previous CDT data (for ARRAY/FOREACH iterating)
-	  @param iIP - instruction pointer
-	  @param oDebugInfo - debug information object
-	  @return instruction pointer if success, -1 if any error occured
-	*/
-	INT_32 ResetForeachScope(const UINT_32        iIP,
-	                         const VMDebugInfo  & oDebugInfo = VMDebugInfo());
-
 	/**
 	  @brief Execute system call such as HREF_PARAM, FORM_PARAM, etc
 	  @param szSyscallName - syscall name
@@ -208,6 +173,14 @@ public:
 	*/
 	INT_32 PushBlockVariable(const UINT_32 iIdx,
 	                         const VMDebugInfo & oDebugInfo = VMDebugInfo());
+
+    /**
+      @brief Prepare local scope for searching variable
+	  @param oDebugInfo - debug information object
+	  @return instruction pointer if success, -1 if any error occured
+    */
+    INT_32 PrepareLocalScope(const VMDebugInfo & oDebugInfo = VMDebugInfo());
+
 	/**
 	  @brief Push variable into stack
 	  @param szVariableName - variable name
@@ -219,15 +192,6 @@ public:
 	                    const UINT_32        iVariableNameLength,
 	                    const VMDebugInfo  & oDebugInfo = VMDebugInfo());
 
-	/**
-	  @brief Push contextual variable into stack
-	  @param iTokenType - type of token
-	  @param oDebugInfo - debug information object
-	  @return instruction pointer if success, -1 if any error occured
-	*/
-	INT_32 PushContextualVariable(const eCTPP2LoopContextVar  & iTokenType,
-	                              const VMDebugInfo           & oDebugInfo = VMDebugInfo());
-
 	/**
 	  @brief Push integer value into stack
 	  @param iVariable - value to push
@@ -284,20 +248,15 @@ public:
 	  @brief Call block by name
 	  @param sBlockName - block name
 	  @param bIsVariable - is true if block name is variable, not plain text
+	  @param iArgCount - number of arguments
 	  @param oDebugInfo - debug information object
 	  @return instruction pointer if success, -1 if any error occured
 	*/
 	INT_32 CallBlock(const STLW::string   & sBlockName,
 	                 const bool           & bIsVariable,
+	                 const UINT_32          iArgCount,
 	                 const VMDebugInfo    & oDebugInfo = VMDebugInfo());
 
-	/**
-	  @brief Call block by __CONTENT__ variable
-	  @param oDebugInfo - debug information object
-	  @return instruction pointer if success, -1 if any error occured
-	*/
-	INT_32 CallContextBlock(const VMDebugInfo & oDebugInfo = VMDebugInfo());
-
 	/**
 	  @brief Start of block
 	  @param sBlockName - block name
@@ -317,9 +276,8 @@ public:
 	/**
 	  @brief Prepare call block
 	  @param oDebugInfo - debug information object
-	  @return instruction pointer if success, -1 if any error occured
 	*/
-	INT_32 PrepareCallBlock(const VMDebugInfo & oDebugInfo = VMDebugInfo());
+	void PrepareCallBlock(const VMDebugInfo & oDebugInfo = VMDebugInfo());
 
 	// ////////////////////////////////////////////////////////////////////////////////
 
@@ -432,22 +390,16 @@ public:
 
 	/**
 	  @brief Push variable into stack
-	  @param szNS - namespace
-	  @param iNSLength - namespace length
 	  @param szVariableName - variable name
 	  @param iVariableNameLength - variable name length
 	  @param oDebugInfo - debug information object
-	  @param bNoPushVariable - don't push variable, but return 0
 	  @return instruction pointer if success, -1 if any error occured
 	*/
-	INT_32 PushScopedVariable(CCHAR_P              szNS,
-	                          const UINT_32        iNSLength,
-	                          CCHAR_P              szName,
+	INT_32 PushScopedVariable(CCHAR_P              szName,
 	                          const UINT_32        iNameLength,
 	                          CCHAR_P              szFullVariable,
 	                          const UINT_32        iFullVariableLength,
-	                          const VMDebugInfo  & oDebugInfo,
-                              bool                 bNoPushVariable);
+	                          const VMDebugInfo  & oDebugInfo);
 
 	/**
 	  @brief Push variable on stack into stack
@@ -459,18 +411,11 @@ public:
 	                         const VMDebugInfo  & oDebugInfo = VMDebugInfo());
 
 	/**
-	  @brief Indirect hash call
+	  @brief Indirect call
 	  @param oDebugInfo - debug information object
 	  @return instruction pointer if success, -1 if any error occured
 	*/
-	INT_32 IndirectHashCall(const VMDebugInfo  & oDebugInfo);
-
-	/**
-	  @brief Indirect array call
-	  @param oDebugInfo - debug information object
-	  @return instruction pointer if success, -1 if any error occured
-	*/
-	INT_32 IndirectArrayCall(const VMDebugInfo  & oDebugInfo);
+	INT_32 IndirectCall(const VMDebugInfo  & oDebugInfo);
 
 	/**
 	  @brief Unconditional jump
@@ -670,6 +615,8 @@ private:
 	UINT_32                            iOneId;
 	/** Current block stack depth  */
 	UINT_32                            iCurrBlockStackDepth;
+	/** Saved stack depths for blocks */
+	STLW::vector<UINT_32>              vSavedStackDepths;
 };
 
 } // namespace CTPP
