@@ -80,12 +80,6 @@ public:
 	*/
 	INT_32 Compile(const UINT_32 & iHalt = 1);
 
-	/**
-	  @brief Set parameter names translation map
-	  @param oIParamTranslationMap - translation map
-	*/
-	void SetParamMap(const IncludeMapType & oIParamTranslationMap);
-
 	/**
 	  @brief A destructor
 	*/
@@ -101,7 +95,7 @@ private:
 	                      TMPL_unless  = 3,
 	                      TMPL_else    = 4,
 	                      TMPL_elsif   = 5,
-	                      TMPL_loop    = 6,
+	                      // TMPL_loop    = 6, // Obsolete
 	                      TMPL_foreach = 7,
 	                      TMPL_include = 8,
 	                      TMPL_call    = 9,
@@ -162,23 +156,6 @@ private:
 	/** Keywords table */
 	static CTPP2Keyword aKeywords[];
 
-	/**
-	  @class CTPP2LoopKeyword CTPP2Parser.hpp <CTPP2Parser.hpp>
-	  @brief CTPP2 reserverd words for loops
-	*/
-	struct CTPP2LoopKeyword
-	{
-		/** keyword                        */
-		CCHAR_P          keyword;
-		/** keyword length                 */
-		UINT_32          keyword_length;
-		/** keyword corresponding operator */
-		UINT_32          keyword_operator;
-	};
-
-	/** Loop modifiers        */
-	static CTPP2LoopKeyword aLoopKeywords [];
-
 	/**
 	  @struct CTPP2Relation CTPP2Parser.hpp <CTPP2Parser.hpp>
 	  @brief CTPP2 Relation
@@ -196,23 +173,6 @@ private:
 	/** Relations */
 	static CTPP2Relation aCTPP2Relations [];
 
-	/**
-	  @struct CTPP2LoopContextVar CTPP2Parser.hpp <CTPP2Parser.hpp>
-	  @brief CTPP2 Loop context variable
-	*/
-	struct CTPP2LoopContextVar
-	{
-		/** keyword                        */
-		CCHAR_P               keyword;
-		/** keyword length                 */
-		UINT_32               keyword_length;
-		/** keyword corresponding operator */
-		eCTPP2LoopContextVar  keyword_operator;
-	};
-
-	/** Loop context variable   */
-	static CTPP2LoopContextVar aLoopContextVars [];
-
 	/** Current operator        */
 	eCTPP2Operator  eBreakFound;
 
@@ -236,9 +196,6 @@ private:
 	/** Static text segment     */
 	StaticText         & oStaticText;
 
-	/** Translation map         */
-	IncludeMapType       oParamTranslationMap;
-
 	/** Template source name    */
 	STLW::string         sSourceName;
 	/** Template source name ID */
@@ -249,10 +206,6 @@ private:
 	INT_32              iRecursionLevel;
 	/** Complex variable flag   */
 	bool                bInsideComplexVariable;
-	/** No push variable on stack */
-	bool                bNoPushVariable;
-	/** No map variable */
-	bool                bNoMapVariable;
 	/** enable/disable verbose mode */
 	bool                bVerboseMode;
 	/** Block flag                  */
@@ -262,6 +215,9 @@ private:
 	/** Map of number of arguments of blocks */
 	BlockArgSizeMapType mBlockArgSizes;
 
+	/** JMP points for TMPL_break */
+	STLW::vector<STLW::vector<INT_32> > vBreakJMPPoints;
+
 	// Simple tokens: open and close tags, operators, variables, strings and numbers //////////////////////////////////////////////////////////////////////////////////
 
 	/**
@@ -281,7 +237,7 @@ private:
 	CCharIterator IsCloseTag(CCharIterator szData, CCharIterator szEnd);
 
 	/**
-	  @brief Operator (var | if | else | unless | loop | foreach | udf | comment | call | break)
+	  @brief Operator (var | if | else | unless | foreach | udf | comment | call | break)
 	  @param szData - current stream position
 	  @param szEnd - end of stream position
 	  @param oOperatorType - operator type
@@ -303,7 +259,6 @@ private:
 	  @brief CTPP2 function a-zA-Z_[a-zA-Z0-9_]
 	  @param szData - current stream position
 	  @param szEnd - end of stream position
-	  @param szNS - namespace, if present
 	  @return NULL if sequence not found or pointer to end of sequence
 	*/
 	CCharIterator IsFuncName(CCharIterator szData, CCharIterator szEnd);
@@ -320,10 +275,9 @@ private:
 	  @brief CTPP2 variable a-zA-Z_[a-zA-Z0-9_|.|:]
 	  @param szData - current stream position
 	  @param szEnd - end of stream position
-	  @param szNS - namespace, if present
 	  @return NULL if sequence not found or pointer to end of sequence
 	*/
-	CCharIterator IsVar(CCharIterator szData, CCharIterator szEnd, CCharIterator & szNS);
+	CCharIterator IsVar(CCharIterator szData, CCharIterator szEnd);
 
 	/**
 	  @brief CTPP2 string "blah-blah \" clah-clah " | 'blah-blah \' clah-clah '
@@ -333,14 +287,6 @@ private:
 	*/
 	CCharIterator IsString(CCharIterator szData, CCharIterator szEnd);
 
-	/**
-	  @brief CTPP2 old fashion string blah-blah.clah-clah
-	  @param szData - current stream position
-	  @param szEnd - end of stream position
-	  @return NULL if sequence not found or pointer to end of sequenc
-	*/
-	CCharIterator IsStringCompatOldVersion(CCharIterator szData, CCharIterator szEnd);
-
 	/**
 	  @brief Number [-]0-9[. [0-9] ] [E [+-] 0-9]
 	  @param szData - current stream position
@@ -356,24 +302,6 @@ private:
 	*/
 	void ParseFloat(CCharIterator & szData, CCharIterator & szEnd);
 
-	/**
-	  @brief CTPP2 Loop contextual variable (__first__ | __last__ | __inner__ | __odd__ | __even__ | __counter__ | __size__ )
-	  @param szData - current stream position
-	  @param szEnd - end of stream position
-	  @param eContextVariable - contextual variable that was found
-	  @return NULL if sequence not found or pointer to end of sequence
-	*/
-	CCharIterator IsContextVar(CCharIterator szData, CCharIterator szEnd, eCTPP2LoopContextVar & eContextVariable);
-
-	/**
-	  @brief Loop operator keyword (__global_vars__ | __context_vars__ | __no_global_vars__ | __no_context_vars__ )
-	  @param szData - current stream position
-	  @param szEnd - end of stream position
-	  @param iLoopKeyword - keyword name
-	  @return NULL if sequence not found or pointer to end of sequence
-	*/
-	CCharIterator IsLoopKeyword(CCharIterator szData, CCharIterator szEnd, UINT_32 & iLoopKeyword);
-
 	/**
 	  @brief Foreach operator keyword (as)
 	  @param szData - current stream position
@@ -521,15 +449,6 @@ private:
 	*/
 	CCharIterator IsUnlessExpr(CCharIterator szData, CCharIterator szEnd, eCTPP2ExprOperator & eResultOperator);
 
-	/**
-	  @brief CTPP2 loop expression; &lt;TMPL_loop LoopExpr&gt;
-	  @param szData - current stream position
-	  @param szEnd - end of stream position
-	  @param iRetPoint - return point instruction number [out]
-	  @return NULL if sequence not found or pointer to end of sequence
-	*/
-	CCharIterator IsLoopExpr(CCharIterator szData, CCharIterator szEnd, UINT_32 & iRetPoint);
-
 	/**
 	  @brief Foreach expression &lt;TMPL_loop ForeachExpr&gt;
 	  @param szData - current stream position
@@ -539,15 +458,6 @@ private:
 	*/
 	CCharIterator IsForeachExpr(CCharIterator szData, CCharIterator szEnd, UINT_32 & iRetPoint);
 
-	/**
-	  @brief Break expression; &lt;TMPL_break BreakExpr&gt;
-	  @param szData - current stream position
-	  @param szEnd - end of stream position
-	  @param iRetPoint - return point instruction number
-	  @return NULL if sequence not found or pointer to end of sequence
-	*/
-	CCharIterator IsBreakExpr(CCharIterator szData, CCharIterator szEnd, UINT_32 & iRetPoint);
-
 	// Operators //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
 	/**
@@ -574,14 +484,6 @@ private:
 	*/
 	CCharIterator UnlessOperator(CCharIterator szData, CCharIterator szEnd);
 
-	/**
-	  @brief Loop operator &lt;TMPL_loop Expr&gt; blah-blah-blah &lt/TMPL_loop&gt;
-	  @param szData - current stream position
-	  @param szEnd - end of stream position
-	  @return NULL if sequence not found or pointer to end of sequence
-	*/
-	CCharIterator LoopOperator(CCharIterator szData, CCharIterator szEnd);
-
 	/**
 	  @brief Foreach operator &lt;TMPL_foreach var_array as var&gt; blah-blah-blah &lt/TMPL_foreach&gt;
 	  @param szData - current stream position
@@ -630,6 +532,13 @@ private:
 	*/
 	CCharIterator VerboseOperator(CCharIterator szData, CCharIterator szEnd);
 
+	/**
+	  @brief Break operator; &lt;TMPL_break&gt;
+	  @param szData - current stream position
+	  @param szEnd - end of stream position
+	  @return NULL if sequence not found or pointer to end of sequence
+	*/
+	CCharIterator BreakOperator(CCharIterator szData, CCharIterator szEnd);
 	// Other stuff ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
 	/**
@@ -640,17 +549,6 @@ private:
 	*/
 	CCharIterator Parse(CCharIterator szData, CCharIterator szEnd);
 
-	/**
-	  @brief CTPP2 include map: map { a:b, c:d }
-	  @param szData - current stream position
-	  @param szEnd - end of stream position
-	  @param oIncludeTranslationMap - translation map
-	  @return NULL if sequence not found or pointer to end of sequence
-	*/
-	CCharIterator IncludeMap(CCharIterator     szData,
-	                         CCharIterator     szEnd,
-	                         IncludeMapType  & oIncludeTranslationMap);
-
 	/**
 	  @brief Store comparison result (if branch)
 	  @param szData - current position in stream
@@ -667,12 +565,6 @@ private:
 	*/
 	UINT_32 StoreUnlessComparisonResult(CCharIterator szData, const eCTPP2ExprOperator & eResultOperator);
 
-	/**
-	  @brief Check translation map
-	  @param sParam - parameter to check
-	*/
-	void CheckParamMap(STLW::string & sParam);
-
 	/**
 	  @brief Get printable operator name
 	  @param oOperator  operator type
