@@ -34,7 +34,6 @@
 #include "CTPP2SyscallFactory.hpp"
 #include "STLFunctional.hpp"
 #include "functions/FnGetText.hpp"
-#include "functions/FnGetText_.hpp"
 
 #include <stdio.h>
 #include <string.h>
@@ -43,7 +42,13 @@
 #include <ctype.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+
+#if defined _MSC_VER
+#include "libgen_win32.h"
+#else
 #include <libgen.h>
+#endif
+
 #include <assert.h>
 
 #define LE_MAGIC 0x950412DE
@@ -219,12 +224,12 @@ static void Split(const STLW::string & sStr, CHAR_8 cSep, STLW::vector<STLW::str
 static void Trim(STLW::string & s)
 {
 	s.erase(
-		STLW::find_if(s.rbegin(), s.rend(), STLW::not1(STLW::ptr_fun<INT_32, INT_32>(STLW::isspace))).base(),
+		STLW::find_if(s.rbegin(), s.rend(), STLW::not1(STLW::ptr_fun<INT_32, INT_32>(::isspace))).base(),
 		s.end());
 
 	s.erase(
 		s.begin(),
-		STLW::find_if(s.begin(), s.end(), STLW::not1(STLW::ptr_fun<INT_32, INT_32>(STLW::isspace))));
+		STLW::find_if(s.begin(), s.end(), STLW::not1(STLW::ptr_fun<INT_32, INT_32>(::isspace))));
 }
 
 //
@@ -380,18 +385,14 @@ STLW::string CTPP2GetText::FindPluralMessage(const STLW::string & sLang, const S
 void CTPP2GetText::SetDefaultDomain(const STLW::string & sDomain) { sDefaultDomain = sDomain; }
 
 //
-// Initialize system calls FnGetText/FnGetText_
+// Initialize system calls FnGetText
 //
 void CTPP2GetText::InitSTDLibFunction(SyscallFactory & oSyscallFactory)
 {
-	{
-		FnGetText * pTMP = dynamic_cast<FnGetText *>(oSyscallFactory.GetHandlerByName("gettext"));
-		pTMP -> SetGetText(this);
-	}
-	{
-		FnGetText_ * pTMP = dynamic_cast<FnGetText_ *>(oSyscallFactory.GetHandlerByName("_"));
-		pTMP -> SetGetText(this);
-	}
+	FnGetText * pTMP = dynamic_cast<FnGetText *>(oSyscallFactory.GetHandlerByName("gettext"));
+	pTMP -> SetGetText(this);
+	pTMP = dynamic_cast<FnGetText *>(oSyscallFactory.GetHandlerByName("_"));
+	pTMP -> SetGetText(this);
 }
 
 //
@@ -399,14 +400,10 @@ void CTPP2GetText::InitSTDLibFunction(SyscallFactory & oSyscallFactory)
 //
 void CTPP2GetText::SetLanguage(SyscallFactory & oSyscallFactory, const STLW::string & sLang)
 {
-	{
-		FnGetText * pTMP = dynamic_cast<FnGetText *>(oSyscallFactory.GetHandlerByName("gettext"));
-		pTMP -> SetLanguage(sLang);
-	}
-	{
-		FnGetText_ * pTMP = dynamic_cast<FnGetText_ *>(oSyscallFactory.GetHandlerByName("_"));
-		pTMP -> SetLanguage(sLang);
-	}
+	FnGetText * pTMP = dynamic_cast<FnGetText *>(oSyscallFactory.GetHandlerByName("gettext"));
+	pTMP -> SetLanguage(sLang);
+	pTMP = dynamic_cast<FnGetText *>(oSyscallFactory.GetHandlerByName("_"));
+	pTMP -> SetLanguage(sLang);
 }
 
 // Parsing of .mo file ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
