@@ -30,9 +30,12 @@
  * $CTPP$
  */
 #include "CTPP2Util.hpp"
+#include "CTPP2Exception.hpp"
 
 #include "CDT.hpp"
 
+#include <stdio.h>
+
 namespace CTPP // C++ Template Engine
 {
 
@@ -140,7 +143,7 @@ STLW::string URLEscape(const STLW::string  & sData)
 		if ((chTMP >= 'a' && chTMP <= 'z') ||
 		    (chTMP >= 'A' && chTMP <= 'Z') ||
 		    (chTMP >= '0' && chTMP <= '9') ||
-		     chTMP == '/' || chTMP == '.' || chTMP == '-')
+		     chTMP == '/' || chTMP == '.' || chTMP == '-' || chTMP == '_')
 		{
 			sBuffer[iBufferPointer++] = *itsData;
 		}
@@ -192,7 +195,7 @@ STLW::string URIEscape(const STLW::string  & sData)
 		if ((chTMP >= 'a' && chTMP <= 'z') ||
 		    (chTMP >= 'A' && chTMP <= 'Z') ||
 		    (chTMP >= '0' && chTMP <= '9') ||
-		     chTMP == '/' || chTMP == '.' || chTMP == '-')
+		     chTMP == '/' || chTMP == '.' || chTMP == '-' || chTMP == '_')
 		{
 			sBuffer[iBufferPointer++] = *itsData;
 		}
@@ -447,7 +450,16 @@ STLW::string WMLEscape(const STLW::string  & sData)
 	{
 		UCHAR_8 chTMP = (UCHAR_8)(*itsData);
 
-		if (chTMP == '"')
+		if (chTMP < ' ')
+		{
+			if (iBufferPointer >= (CTPP_ESCAPE_BUFFER_LEN - 6))
+			{
+				sRetVal.append(sBuffer, iBufferPointer);
+				iBufferPointer = 0;
+			}
+			snprintf(sBuffer + iBufferPointer, 6, "\\u%04d", chTMP);
+		}
+		else if (chTMP == '"')
 		{
 			if (iBufferPointer >= (CTPP_ESCAPE_BUFFER_LEN - 6))
 			{
@@ -546,105 +558,102 @@ STLW::string WMLEscape(const STLW::string  & sData)
 return sRetVal;
 }
 
+
 //
 // Escape string, if need
 //
 STLW::string EscapeJSONString(const STLW::string & sSource, const bool & bECMAConventions, const bool & bHTMLSafe)
 {
-	STLW::string sResult;
+    DumpBuffer oBuff;
+    DumpJSONString(oBuff, sSource, bECMAConventions, bHTMLSafe);
+    return STLW::string(oBuff.Data(), oBuff.Size());
+}
 
-	const UINT_32 iSize = sSource.size();
-	if (iSize == 0) { return sResult; }
 
-	UINT_32 iPos     = 0;
-	UINT_32 iPrevPos = 0;
-	for(;;)
-	{
-		const UCHAR_8 uCH = sSource[iPos];
-		if (uCH == '\r')
-		{
-			sResult.append(sSource, iPrevPos, iPos - iPrevPos);
-			iPrevPos = iPos + 1;
-			sResult.append("\\r", 2);
-		}
-		else if (uCH == '\n')
-		{
-			sResult.append(sSource, iPrevPos, iPos - iPrevPos);
-			iPrevPos = iPos + 1;
-			sResult.append("\\n", 2);
-		}
-		else if (uCH == '\b')
-		{
-			sResult.append(sSource, iPrevPos, iPos - iPrevPos);
-			iPrevPos = iPos + 1;
-			sResult.append("\\b", 2);
-		}
-		else if (uCH == '\f')
-		{
-			sResult.append(sSource, iPrevPos, iPos - iPrevPos);
-			iPrevPos = iPos + 1;
-			sResult.append("\\f", 2);
-		}
-		else if (uCH == '\t')
-		{
-			sResult.append(sSource, iPrevPos, iPos - iPrevPos);
-			iPrevPos = iPos + 1;
-			sResult.append("\\t", 2);
-		}
-		else if (uCH == '\\')
-		{
-			sResult.append(sSource, iPrevPos, iPos - iPrevPos);
-			iPrevPos = iPos + 1;
-			sResult.append("\\\\", 2);
-		}
-		else if (uCH == '"')
-		{
-			sResult.append(sSource, iPrevPos, iPos - iPrevPos);
-			iPrevPos = iPos + 1;
-			sResult.append("\\\"", 2);
-		}
-		else if (bECMAConventions)
-		{
-			if (uCH == '\'')
-			{
-				sResult.append(sSource, iPrevPos, iPos - iPrevPos);
-				iPrevPos = iPos + 1;
-				sResult.append("\\\'", 2);
-			}
-			else if (uCH == '\a')
-			{
-				sResult.append(sSource, iPrevPos, iPos - iPrevPos);
-				iPrevPos = iPos + 1;
-				sResult.append("\\a", 2);
-			}
-			else if (uCH == '\v')
-			{
-				sResult.append(sSource, iPrevPos, iPos - iPrevPos);
-				iPrevPos = iPos + 1;
-				sResult.append("\\v", 2);
-			}
-		}
-		else if (bHTMLSafe)
-		{
-			if (uCH == '<')
-			{
-				sResult.append(sSource, iPrevPos, iPos - iPrevPos);
-				iPrevPos = iPos + 1;
-				sResult.append("\\u003c", 6);
-			}
-			else if (uCH == '>')
-			{
-				sResult.append(sSource, iPrevPos, iPos - iPrevPos);
-				iPrevPos = iPos + 1;
-				sResult.append("\\u003e", 6);
-			}
-		}
+//
+// Escape and dump string to buffer
+//
+DumpBuffer & DumpJSONString(DumpBuffer & sResult, const STLW::string & sSource, const bool & bECMAConventions, const bool & bHTMLSafe)
+{
+    INT_32 iPos = 0;
+    INT_32 iSize = sSource.size();
+    INT_32 iOffset = 0;
 
-		++iPos;
-		if (iPos == iSize) { break; }
-	}
+    while (iSize--)
+    {
+        const UCHAR_8 uCH = sSource[iPos];
+        const CHAR_8 * sEscaped = NULL;
+        bool bFound = false;
 
-	if (iPos != iPrevPos) { sResult.append(sSource, iPrevPos, iPos - iPrevPos); }
+        switch (uCH)
+        {
+            case '"':
+                sEscaped = "\\\"";
+                bFound = true;
+                break;
+            case '\\':
+                sEscaped = "\\\\";
+                bFound = true;
+                break;
+            case '/':
+                sEscaped = "\\/";
+                bFound = true;
+                break;
+            case '\b':
+                sEscaped = "\\b";
+                bFound = true;
+                break;
+            case '\f':
+                sEscaped = "\\f";
+                bFound = true;
+                break;
+            case '\n':
+                sEscaped = "\\n";
+                bFound = true;
+                break;
+            case '\r':
+                sEscaped = "\\r";
+                bFound = true;
+                break;
+            case '\t':
+                sEscaped = "\\t";
+                bFound = true;
+                break;
+            case '\'':
+                sEscaped = "\\'";
+                if (bECMAConventions) { bFound = true; }
+                break;
+            case '\v':
+                sEscaped = "\\v";
+                if (bECMAConventions) { bFound = true; }
+                break;
+            case '\0':
+                sEscaped = "\\0";
+                if (bECMAConventions) { bFound = true; }
+                break;
+            default:
+                break;
+        }
+
+        if (bFound)
+        {
+            if (iPos - iOffset > 0) { sResult.Write(sSource.c_str() + iOffset, iPos - iOffset); }
+            sResult.Write(sEscaped, strlen(sEscaped));
+            iOffset = iPos + 1;
+        }
+        else if (uCH < ' ' || (bHTMLSafe && (uCH == '<' || uCH == '>')))
+        {
+            if (iPos - iOffset > 0) { sResult.Write(sSource.c_str() + iOffset, iPos - iOffset); }
+            CHAR_8 szBuffer[7] = { '\0' };
+            const CHAR_8 *szHexChars = "0123456789abcdef";
+			snprintf(szBuffer, 7, "\\u00%c%c", szHexChars[uCH >> 4], szHexChars[uCH & 0xF]);
+			sResult.Write(szBuffer, 6);
+            iOffset = iPos + 1;
+        }
+        ++iPos;
+    }
+
+    if (iPos - iOffset > 0) { sResult.Write(sSource.c_str() + iOffset, iPos - iOffset); }
 
 return sResult;
 }
@@ -653,12 +662,59 @@ return sResult;
 // Dump CDT to JSON
 //
 void CDT2JSON(const CTPP::CDT & oCDT, STLW::string & sData)
+{
+    DumpBuffer oBuff;
+    DumpCDT2JSON(oCDT, oBuff);
+    sData.append(oBuff.Data(), oBuff.Size());
+}
+
+DumpBuffer::~DumpBuffer()
+{
+	free(pBuffer);
+}
+
+DumpBuffer::DumpBuffer()
+{
+	pBuffer = 0;
+	iCapacity = 0;
+	pPos = 0;
+}
+
+CCHAR_P DumpBuffer::Data() const { return pBuffer; }
+
+DumpBuffer::StreamSize DumpBuffer::Size() const { return pPos - pBuffer; }
+
+void DumpBuffer::Reserve(DumpBuffer::StreamSize iN)
+{
+	if (iN <= iCapacity) { return; }
+
+	StreamSize iOffset = pPos - pBuffer;
+	CHAR_P pTmp = (CHAR_P) realloc(pBuffer, iN);
+	if(!pTmp) { throw CTPPNoMemoryError(); }
+	pBuffer = pTmp;
+
+	pPos = pBuffer + iOffset;
+	iCapacity = iN;
+}
+
+void DumpBuffer::Write(CCHAR_P pStr, DumpBuffer::StreamSize iN) {
+	if(pPos + iN > pBuffer + iCapacity) {
+		Reserve(2 * (iCapacity + iN));
+	}
+	memcpy(pPos, pStr, iN);
+	pPos += iN;
+}
+
+//
+// Escape and dump string to buffer
+//
+DumpBuffer & DumpCDT2JSON(const CTPP::CDT & oCDT, DumpBuffer & oResult)
 {
 	using namespace CTPP;
 	switch (oCDT.GetType())
 	{
 		case CDT::UNDEF:
-			sData.append("null");
+			oResult.Write("null", 4);
 			break;
 
 		case CDT::INT_VAL:
@@ -666,48 +722,53 @@ void CDT2JSON(const CTPP::CDT & oCDT, STLW::string & sData)
 		case CDT::POINTER_VAL:
 		case CDT::STRING_INT_VAL:
 		case CDT::STRING_REAL_VAL:
-			sData.append(oCDT.GetString());
+            {
+				STLW::string tmp = oCDT.GetString();
+				oResult.Write(tmp.data(), tmp.size());
+			}
 			break;
 
 		case CDT::STRING_VAL:
-			sData.append("\"");
-			sData.append(EscapeJSONString(oCDT.GetString(), false));
-			sData.append("\"");
+			oResult.Write("\"", 1);
+			DumpJSONString(oResult, oCDT.GetString(), false);
+			oResult.Write("\"", 1);
 			break;
 
 		case CDT::ARRAY_VAL:
 			{
-				sData.append("[");
+				oResult.Write("[", 1);
 				UINT_32 iJ = 0;
-				while(iJ < oCDT.Size())
+				while (iJ < oCDT.Size())
 				{
-					CDT2JSON(oCDT.GetCDT(iJ), sData);
+					DumpCDT2JSON(oCDT.GetCDT(iJ), oResult);
 					++iJ;
-					if (iJ != oCDT.Size()) { sData.append(","); }
+					if (iJ != oCDT.Size()) { oResult.Write(",", 1); }
 				}
-				sData.append("]");
+				oResult.Write("]", 1);
 			}
 			break;
 
 		case CDT::HASH_VAL:
 			{
-				sData.append("{");
+				oResult.Write("{", 1);
 				CDT::ConstIterator itCDTCArray = oCDT.Begin();
 				while (itCDTCArray != oCDT.End())
 				{
-					sData.append("\"");
-					sData.append(EscapeJSONString(itCDTCArray -> first, false));
-					sData.append("\":");
+					oResult.Write("\"", 1);
+					DumpJSONString(oResult, itCDTCArray -> first, false);
+					oResult.Write("\":", 2);
 
-					CDT2JSON(itCDTCArray -> second, sData);
+					DumpCDT2JSON(itCDTCArray -> second, oResult);
 
 					++itCDTCArray;
-					if (itCDTCArray != oCDT.End()) { sData.append(","); }
+					if (itCDTCArray != oCDT.End()) { oResult.Write(",", 1); }
 				}
-				sData.append("}");
+				oResult.Write("}", 1);
 			}
 			break;
 	}
+
+return oResult;
 }
 
 //
