@@ -306,11 +306,21 @@ HL_RST;
 									else if (iSrcReg == ARG_SRC_STACK)
 									{
 #ifdef _DEBUG
-fprintf(stderr, "STACK[%d] (`%s`)\n", 0, oVMArgStack.GetTopElement(0).GetString().c_str());
+fprintf(stderr, "STACK[%d] (`%s`)\n", aCode[iIP].argument, oVMArgStack.GetTopElement(aCode[iIP].argument).GetString().c_str());
 HL_RST;
 #endif
-										sCallName = oVMArgStack.GetTopElement(0).GetString();
-										oVMArgStack.ClearStack(1);
+										sCallName = oVMArgStack.GetTopElement(aCode[iIP].argument).GetString();
+										STLW::vector<CDT> vArgs;
+										vArgs.reserve(aCode[iIP].argument);
+										for (UINT_32 iI = 0; iI < aCode[iIP].argument; ++iI)
+										{
+											vArgs.push_back(oVMArgStack.GetTopElement(iI));
+										}
+										oVMArgStack.ClearStack(aCode[iIP].argument + 1);
+										for (STLW::vector<CDT>::reverse_iterator vIt = vArgs.rbegin(); vIt != vArgs.rend(); ++vIt)
+										{
+											oVMArgStack.PushElement(*vIt);
+										}
 									}
 									// Illegal Opcode?
 									else
@@ -545,61 +555,6 @@ HL_RST;
 								}
 								break;
 
-							// RLOOP, cycle
-							case SYSCALL_OPCODE_LO(RLOOP):
-								{
-									// Check execution limit
-									if (iExecutedSteps >= iMaxSteps)
-									{
-										UINT_32 iDataSize = 0;
-										CCHAR_P szTMP     = pMemoryCore -> static_text.GetData(VMDebugInfo(aCode[iIP].reserved).GetDescrId(), iDataSize);
-										throw ExecutionLimitReached(iIP, aCode[iIP].reserved, szTMP);
-									}
-
-									const UINT_32 iSrcReg = SYSCALL_REG_SRC(iOpCode);
-									const UINT_32 iDstReg = SYSCALL_REG_DST(iOpCode);
-									if (iDstReg <= ARG_DST_LASTREG && iSrcReg <= ARG_SRC_LASTREG)
-									{
-										CDT & oLoopReg = oRegs[iSrcReg];
-
-#ifdef _DEBUG
-HL_CODE(GREEN);
-fprintf(stderr, "0x%08X RLOOP     0x%08X (%cR = %d, %cR = %d)\n", iIP, aCode[iIP].argument, CHAR_8(iSrcReg + 'A'), INT_32(oLoopReg.GetInt()), CHAR_8((iDstReg >> 8) + 'A'), INT_32(oRegs[iDstReg >> 8].GetInt()));
-HL_RST;
-#endif
-
-										// Decrease number of iterations
-										--oLoopReg;
-										// End of cycle?
-										if (oLoopReg <= 0) { ++iIP; }
-										// New iteration
-										else
-										{
-											const UINT_32 iNewIP = iIP - aCode[iIP].argument;
-											// New IP is correct?
-											if (iNewIP >= iCodeLength)
-											{
-												UINT_32 iDataSize = 0;
-												CCHAR_P szTMP     = pMemoryCore -> static_text.GetData(VMDebugInfo(aCode[iIP].reserved).GetDescrId(), iDataSize);
-												throw CodeSegmentOverrun(iIP, aCode[iIP].reserved, szTMP);
-											}
-											iIP = iNewIP;
-
-											// Iteration counter
-											++oRegs[iDstReg >> 8];
-										}
-									}
-									// Illegal Opcode?
-									else
-									{
-										UINT_32 iDataSize = 0;
-										CCHAR_P szTMP     = pMemoryCore -> static_text.GetData(VMDebugInfo(aCode[iIP].reserved).GetDescrId(), iDataSize);
-										throw IllegalOpcode(iIP, iOpCode, aCode[iIP].reserved, szTMP);
-									}
-								}
-
-								break;
-
 							// Illegal Opcode?
 							default:
 							{
@@ -1459,13 +1414,52 @@ HL_RST;
 												CCHAR_P szTMP     = pMemoryCore -> static_text.GetData(VMDebugInfo(aCode[iIP].reserved).GetDescrId(), iDataSize);
 												throw IllegalOpcode(iIP, iOpCode, aCode[iIP].reserved, szTMP);
 											}
+											// AR <- CR[BR]
+											//oRegs[iDstReg >> 8] = oRegs[iSrcReg][oRegs[iArgNum].GetInt()];
 #ifdef _DEBUG
 HL_CODE(GREEN);
-fprintf(stderr, "0x%08X MOVIREG   %cR, %cR[%cR] (%d): %s\n", iIP, CHAR_8((iDstReg >> 8) + 'A'), CHAR_8(iSrcReg + 'A'), CHAR_8(iArgNum + 'A'), INT_32(oRegs[iArgNum].GetInt()), oRegs[iSrcReg][oRegs[iArgNum].GetInt()].GetString().c_str());
+fprintf(stderr, "0x%08X MOVIREG   %cR, %cR[%cR] ", iIP, CHAR_8((iDstReg >> 8) + 'A'), CHAR_8(iSrcReg + 'A'), CHAR_8(iArgNum + 'A'));
+#endif
+                                            const INT_32 iIdx = oRegs[iArgNum].GetInt();
+                                            CDT oItVal(CDT::HASH_VAL);
+                                            if (iIdx == 0)
+                                            {
+                                                oItVal["__first__"] = 1;
+                                            }
+                                            else if (UINT_32(iIdx + 1) == oRegs[iSrcReg].Size())
+                                            {
+                                                oItVal["__last__"] = 1;
+                                            }
+                                            else
+                                            {
+                                                oItVal["__inner__"] = 1;
+                                            }
+
+                                            if ((iIdx + 1) % 2 == 1) { oItVal["__odd__"] = 1; }
+                                            else                     { oItVal["__even__"] = 1; }
+
+                                            if (oRegs[iSrcReg].GetType() == CDT::HASH_VAL)
+                                            {
+                                                CDT::Iterator it = oRegs[iSrcReg].Begin();
+                                                for (INT_32 iI = 0; iI < iIdx; ++iI) { ++it; }
+#ifdef _DEBUG
+fprintf(stderr, "(`%s`): %s\n", it->first.c_str(), it->second.GetString().c_str());
 HL_RST;
 #endif
-											// AR <- CR[BR]
-											oRegs[iDstReg >> 8] = oRegs[iSrcReg][oRegs[iArgNum].GetInt()];
+                                                oItVal["__value__"] = it->second;
+                                                oItVal["__key__"] = it->first;
+                                                oRegs[iDstReg >> 8] = oItVal;
+                                            }
+                                            else
+                                            {
+#ifdef _DEBUG
+fprintf(stderr, "(%d): %s\n", iIdx, oRegs[iSrcReg][iIdx].GetString().c_str());
+HL_RST;
+#endif
+                                                oItVal["__value__"] = oRegs[iSrcReg][iIdx];
+                                                oItVal["__index__"] = iIdx;
+                                                oRegs[iDstReg >> 8] = oItVal;
+                                            }
 										}
 										// Illegal Opcode?
 										else
@@ -2118,6 +2112,85 @@ HL_RST;
 									}
 								}
 								break;
+							// REPLIND
+							case SYSCALL_OPCODE_LO(REPLIND):
+								{
+#ifdef _DEBUG
+HL_CODE(RED);
+fprintf(stderr, "0x%08X REPLIND   ", iIP);
+#endif
+									const UINT_32 iSrcReg = SYSCALL_REG_SRC(iOpCode);
+									if (iSrcReg == ARG_SRC_STACK)
+									{
+										CDT::eValType typ = oVMArgStack.GetTopElement(aCode[iIP].argument).GetType();
+										switch (typ)
+										{
+										case CDT::INT_VAL:
+										case CDT::REAL_VAL:
+#ifdef _DEBUG
+fprintf(stderr, "STACK[%d](%d)\n", aCode[iIP].argument, INT_32(oVMArgStack.GetTopElement(aCode[iIP].argument).GetInt()));
+HL_RST;
+#endif
+											oVMArgStack.GetTopElement(0) = oVMArgStack.GetTopElement(0)[oVMArgStack.GetTopElement(aCode[iIP].argument).GetInt()];
+											break;
+										case CDT::STRING_VAL:
+										case CDT::STRING_INT_VAL:
+										case CDT::STRING_REAL_VAL:
+#ifdef _DEBUG
+fprintf(stderr, "STACK[%d](`%s`)\n", aCode[iIP].argument, oVMArgStack.GetTopElement(aCode[iIP].argument).GetString().c_str());
+HL_RST;
+#endif
+											oVMArgStack.GetTopElement(0) = oVMArgStack.GetTopElement(0)[oVMArgStack.GetTopElement(aCode[iIP].argument).GetString()];
+											break;
+										default:
+#ifdef _DEBUG
+fprintf(stderr, "STACK[%d](UNDEF)\n", aCode[iIP].argument);
+HL_RST;
+#endif
+											oVMArgStack.GetTopElement(0) = CDT();
+											break;
+										}
+									}
+									else if (iSrcReg <= ARG_SRC_LASTREG)
+									{
+										CDT::eValType typ = oRegs[iSrcReg].GetType();
+										switch (typ)
+										{
+										case CDT::INT_VAL:
+										case CDT::REAL_VAL:
+#ifdef _DEBUG
+fprintf(stderr, "%cR (%d)\n", CHAR_8(iSrcReg + 'A'), INT_32(oRegs[iSrcReg].GetInt()));
+HL_RST;
+#endif
+											oVMArgStack.GetTopElement(0) = oVMArgStack.GetTopElement(0)[oRegs[iSrcReg].GetInt()];
+											break;
+										case CDT::STRING_VAL:
+										case CDT::STRING_INT_VAL:
+										case CDT::STRING_REAL_VAL:
+#ifdef _DEBUG
+fprintf(stderr, "%cR (`%s`)\n", CHAR_8(iSrcReg + 'A'), oRegs[iSrcReg].GetString().c_str());
+HL_RST;
+#endif
+											oVMArgStack.GetTopElement(0) = oVMArgStack.GetTopElement(0)[oRegs[iSrcReg].GetString()];
+											break;
+										default:
+#ifdef _DEBUG
+fprintf(stderr, "%cR (UNDEF)\n", CHAR_8(iSrcReg + 'A'));
+HL_RST;
+#endif
+											oVMArgStack.GetTopElement(0) = CDT();
+											break;
+										}
+									}
+									// Illegal Opcode?
+									else
+									{
+										UINT_32 iDataSize = 0;
+										CCHAR_P szTMP = pMemoryCore -> static_text.GetData(VMDebugInfo(aCode[iIP].reserved).GetDescrId(), iDataSize);
+										throw IllegalOpcode(iIP, iOpCode, aCode[iIP].reserved, szTMP);
+									}
+								}
+								break;
 							// XCHG
 							case SYSCALL_OPCODE_LO(XCHG):
 								{
@@ -2206,9 +2279,15 @@ HL_RST;
 								{
 #ifdef _DEBUG
 HL_CODE(GREEN);
-fprintf(stderr, "0x%08X SAVEBP\n", iIP);
+fprintf(stderr, "0x%08X SAVEBP", iIP);
+if (aCode[iIP].argument > 0) {
+	fprintf(stderr, " [-%d]\n", aCode[iIP].argument);
+} else {
+	fprintf(stderr, "\n");
+}
+
 #endif
-									oVMArgStack.SaveBasePointer();
+									oVMArgStack.SaveBasePointer(aCode[iIP].argument);
 								}
 								break;
 							// RESTBP, restore base pointer
